from enum import Enum


class SpadlAction(Enum):
    PASS = 'pass'
    CROSS = 'cross'
    THROW_IN = 'throw_in'
    FREEKICK_CROSSED = 'freekick_crossed'
    FREEKICK_SHORT = 'freekick_short'
    CORNER_CROSSED = 'corner_crossed'
    CORNER_SHORT = 'corner_short'
    TAKE_ON = 'take_on'
    FOUL = 'foul'
    TACKLE = 'tackle'
    INTERCEPTION = 'interception'
    SHOT = 'shot'
    SHOT_PENALTY = 'shot_penalty'
    SHOT_FREEKICK = 'shot_freekick'
    KEEPER_SAVE = 'keeper_save'
    KEEPER_CLAIM = 'keeper_claim'
    KEEPER_PUNCH = 'keeper_punch'
    KEEPER_PICK_UP = 'keeper_pick_up'
    CLEARANCE = 'clearance'
    BAD_TOUCH = 'bad_touch'
    NON_ACTION = 'non_action'
    DRIBBLE = 'dribble'
    GOALKICK = 'goalkick'
    RECEIVAL = 'receival'
    OUT = 'out'
    OFFSIDE = 'offside'
    GOAL = 'goal'
    OWN_GOAL = 'owngoal'
    YELLOW_CARD = 'yellow_card'
    RED_CARD = 'red_card'
    CORNER = 'corner'
    FREEKICK = 'freekick'


class SpadlBodyPart(Enum):
    HEAD = 'head'
    FOOT = 'foot'
    OTHER = 'other'
    HEAD_OTHER = 'head_other'
    LEFT_FOOT = 'foot_left'
    RIGHT_FOOT = 'foot_right'
